@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2009-2012, 2014-2017 Free Software Foundation, Inc.
+   Copyright (C) 2009-2020 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -25,49 +25,18 @@ _repr (PyObject *self)
 {
   char buf[80];
   sprintf (buf, "<" PY_MODULE "." PY_CSNAME " instance at %p>", self);
-  return PyString_FromString (buf);
+  return PyUnicode_FromString (buf);
 }
 
 static PyTypeObject PyMimeType = {
-  PyObject_HEAD_INIT(NULL)
-  0,                         /* ob_size */
-  PY_MODULE "." PY_CSNAME,   /* tp_name */
-  sizeof (PyMime),           /* tp_basicsize */
-  0,                         /* tp_itemsize */
-  (destructor)_py_dealloc,   /* tp_dealloc */
-  0,                         /* tp_print */
-  0,                         /* tp_getattr; __getattr__ */
-  0,                         /* tp_setattr; __setattr__ */
-  0,                         /* tp_compare; __cmp__ */
-  _repr,                     /* tp_repr; __repr__ */
-  0,                         /* tp_as_number */
-  0,                         /* tp_as_sequence */
-  0,                         /* tp_as_mapping */
-  0,                         /* tp_hash; __hash__ */
-  0,                         /* tp_call; __call__ */
-  _repr,                     /* tp_str; __str__ */
-  0,                         /* tp_getattro */
-  0,                         /* tp_setattro */
-  0,                         /* tp_as_buffer */
-  Py_TPFLAGS_DEFAULT,        /* tp_flags */
-  "",                        /* tp_doc */
-  0,                         /* tp_traverse */
-  0,                         /* tp_clear */
-  0,                         /* tp_richcompare */
-  0,                         /* tp_weaklistoffset */
-  0,                         /* tp_iter */
-  0,                         /* tp_iternext */
-  0,                         /* tp_methods */
-  0,                         /* tp_members */
-  0,                         /* tp_getset */
-  0,                         /* tp_base */
-  0,                         /* tp_dict */
-  0,                         /* tp_descr_get */
-  0,                         /* tp_descr_set */
-  0,                         /* tp_dictoffset */
-  0,                         /* tp_init */
-  0,                         /* tp_alloc */
-  0,                         /* tp_new */
+  .ob_base = { PyObject_HEAD_INIT(NULL) },
+  .tp_name = PY_MODULE "." PY_CSNAME,
+  .tp_basicsize = sizeof (PyMime),
+  .tp_dealloc = (destructor)_py_dealloc,
+  .tp_repr = _repr,
+  .tp_str = _repr,
+  .tp_flags = Py_TPFLAGS_DEFAULT,
+  .tp_doc = "",
 };
 
 PyMime *
@@ -94,7 +63,7 @@ api_mime_create (PyObject *self, PyObject *args)
     }
 
   status = mu_mime_create (&py_mime->mime, py_msg->msg, flags);
-  return _ro (PyInt_FromLong (status));
+  return _ro (PyLong_FromLong (status));
 }
 
 static PyObject *
@@ -133,7 +102,7 @@ api_mime_get_num_parts (PyObject *self, PyObject *args)
     return NULL;
 
   status = mu_mime_get_num_parts (py_mime->mime, &nparts);
-  return status_object (status, PyInt_FromSize_t (nparts));
+  return status_object (status, PyLong_FromSize_t (nparts));
 }
 
 static PyObject *
@@ -170,7 +139,7 @@ api_mime_add_part (PyObject *self, PyObject *args)
     }
 
   status = mu_mime_add_part (py_mime->mime, py_msg->msg);
-  return _ro (PyInt_FromLong (status));
+  return _ro (PyLong_FromLong (status));
 }
 
 static PyObject *
@@ -200,7 +169,7 @@ api_rfc2047_decode (PyObject *self, PyObject *args)
     return NULL;
 
   status = mu_rfc2047_decode (tocode, text, &buf);
-  return status_object (status, PyString_FromString (mu_prstr (buf)));
+  return status_object (status, PyUnicode_FromString (mu_prstr (buf)));
 }
 
 static PyObject *
@@ -214,7 +183,7 @@ api_rfc2047_encode (PyObject *self, PyObject *args)
     return NULL;
 
   status = mu_rfc2047_encode (charset, encoding, text, &buf);
-  return status_object (status, PyString_FromString (mu_prstr (buf)));
+  return status_object (status, PyUnicode_FromString (mu_prstr (buf)));
 }
 
 static PyMethodDef methods[] = {
@@ -248,6 +217,14 @@ static PyMethodDef methods[] = {
   { NULL, NULL, 0, NULL }
 };
 
+static struct PyModuleDef moduledef = {
+  PyModuleDef_HEAD_INIT,
+  PY_MODULE,
+  NULL,
+  -1,
+  methods
+};
+
 int
 mu_py_init_mime (void)
 {
@@ -259,7 +236,7 @@ void
 _mu_py_attach_mime (void)
 {
   PyObject *m;
-  if ((m = _mu_py_attach_module (PY_MODULE, methods)))
+  if ((m = _mu_py_attach_module (&moduledef)))
     {
       Py_INCREF (&PyMimeType);
       PyModule_AddObject (m, PY_CSNAME, (PyObject *)&PyMimeType);
